/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * license.p
 *
 *
 * Show license info from "showcfg" -- not well tested.  Especially on Windows and on ancient, obsolete and unsupported platforms
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	August 17, 2019
 *
 */

{lib/protop.i}

define output parameter dcDescription as character no-undo initial "LicenseCfg".

define temp-table tt_license no-undo
  field xid          as integer   format ">>9"     label "Id"
  field productName  as character format "x(40)"   label "Product Name"
  field installDate  as character format "x(11)"   label "  Installed"
  field fluff        as character format "x(3)"    label ""
  field expireDate   as character format "x(11)"   label "    Expires"
  field fluff2       as character format "x(3)"    label ""
  field serialNum    as character format "x(9)"    label "  Serial#"
  field zControl     as character format "x(24)"   label "Control Codes"
  field fluff3       as character format "x(3)"    label ""
  field pscVersion   as character format "x(8)"    label " Version"
  field userLimit    as integer   format "->>>>>9" label "Users"
  field machineClass as character format "x(8)"    label "   Class"		/* need a list along with translations, EB, KB, Z1...	*/
  field osPortCode   as integer   format ">>>>>>9" label "Port"
  field osPortText   as character format "x(35)"   label ""

  index xid-idx      is unique xid
.

define temp-table tt_license_Info no-undo
  field infoString as character
  index infoString-idx is primary unique infoString
.

define variable configFile  as character no-undo.
define variable companyName as character no-undo.

/* https://knowledgebase.progress.com/articles/Article/P61158
 *
 * HP 11 32 bit has 173 CD and 196 DVD media_id and port_number = 34
 * HP 11 64 bit has 175 CD and 198 DVD media_id and port_number = 36 
 * HP Itanium2 has 186 CD and 197 DVD media_id and port_number = 40 
 * Sun Solaris SPARC 32 bit has 157 CD and 202 DVD media_id and port_number = 11
 * Sun Solaris SPARC 64 bit has 182 CD and 203 DVD media_id and port_number = 39
 * IBM AIX POWER 32 bit has 156 CD and 200 DVD media_id and port_number = 17
 * IBM AIX POWER 64 bit has 187 CD and 201 DVD media_id and port_number = 37
 * LINUX POWERPC 64 bit has 209 CD and 210 DVD media_id and port_number = 45
 * LINUX AMD 64 bit has 192 CD and 205 DVD media_id and port_number = 43
 * LINUX INTEL 32 bit has 178 CD and 204 DVD media_id and port_number = 38
 * WINDOWS x86 32 bit has 146 CD and 195 DVD media_id and port_number = 31
 * WINDOWS x86 64 bit has 212 CD and 218 DVD media_id and port_number = 33
 */

define variable portCodeList as character no-undo.
define variable portTextList as character no-undo.

portCodeList = "34,36,40,11,39,17,37,45,43,38,31,33".
portTextList = "HPUX 32bit,HPUX 64bit,Itanium2,Sparc32,Sparc64,AIX32,AIX64,Linux PowerPC 64,Linux64,Linux32,Windows32,Windows64".

procedure mon-init:

  define variable b as handle.

  b = ( temp-table tt_license:default-buffer-handle ).

  if pt_altWidth = "80" then
    do:

      altFormat( "80", b, "productName",  "x(30)" ).
      altFormat( "80", b, "expireDate",   "x(11)" ).
      altFormat( "80", b, "pscVersion",   "x(8)" ).
      altFormat( "80", b, "userLimit",    ">>>>9" ).
      altFormat( "80", b, "machineClass", "x(8)" ).
      altFormat( "80", b, "osPortText",   "x(13)" ).

      b:buffer-field( "osPortText" ):label = "Port".

    end.

  return.

end.


define variable lastUpdate as datetime no-undo.

procedure mon-update:

  define input parameter argList as character no-undo.

  define variable inLine    as character no-undo.
  define variable lineType  as character no-undo.
  define variable lineValue as character no-undo.

  define variable prodNum   as integer no-undo.
  define variable i         as integer no-undo.

  define variable b as handle.

  b = ( temp-table tt_license:default-buffer-handle ).

  if lastUpdate = ? or interval( lastUpdate, now, "hours" ) > 1 then
    lastupDate = now.
   else
    do:
      add2ds( temp-table tt_license:default-buffer-handle ).
      return.
    end.

  empty temp-table tt_license.

  if opsys = "unix" then
    input stream inStrm through value( "showcfg" ).		/* -B adds some interesting info	*/
   else
    input stream inStrm through value( "showcfgd" ).

  repeat:

    import stream inStrm unformatted inLIne.

    if inLine = ? or inLine = "" then next.

    lineType  = trim( entry( 1, inLine, ":" )).
    lineValue = trim( substring( inLine, index( inLine, ":" ) + 1 )).

    case lineType:

      when "Configuration File" then
        configFile = lineValue.

      when "Company Name" then
        companyName = lineValue.

      when "Product Name" then
        do:
          prodNum = prodNum + 1.
          create tt_license.
          assign
            tt_license.xid         = prodNum
            tt_license.productName = lineValue
          .
        end.

      when "Installation Date" then tt_license.installDate  = substring( lineValue, 5, 6 ) + substring( lineValue, 20, 5 ).
      when "Expiration Date"   then tt_license.expireDate   = leftPad( lineValue, integer( b:buffer-field( "expireDate" ):width ), " " ).
      when "User Limit"        then tt_license.userLimit    = integer( lineValue ) no-error.
      when "Serial Number"     then tt_license.serialNum    = lineValue.
      when "Control Numbers"   then tt_license.zControl     = lineValue.
      when "Version Number"    then tt_license.pscVersion   = leftPad( lineValue, integer( b:buffer-field( "pscVersion" ):width ), " " ).
      when "Machine Class"     then tt_license.machineClass = leftPad( lineValue, integer( b:buffer-field( "machineClass" ):width ), " " ).
      when "Port Number"       then
        do:
          tt_license.osPortCode = integer( lineValue ) no-error.
          i = lookup( lineValue, portCodeList ).
          tt_license.osPortText = entry( i, portTextList ).
        end.

    end.

  end.
  input stream inStrm close.

  find first tt_license_Info no-error.
  if not available tt_license_Info then
    create tt_license_Info.
  tt_license_Info.infoString = substitute( "PROCFG: &1; Licensed to: &2", configFile, companyName ).

  add2ds( temp-table tt_license:default-buffer-handle ).
  add2ds( temp-table tt_license_Info:default-buffer-handle ).

  return.

end.

return.
