/* userTableStats.p
 *
 * track userTableStats for the current session
 *
 */

/* the public temp tables
 */

{lib/userstats.i}

/* internal temp tables
 */

define temp-table tt_usrTblInfoX no-undo
  field xId     as integer
  field tblNum  as integer
  field tblName as character format "x(20)"
  field tblRd   as int64 extent 2			/* [1] = previous value, [2] = incremental value	*/
  field tblCr   as int64 extent 2    
  field tblUp   as int64 extent 2
  field tblDl   as int64 extent 2
  index tblNum-idx  is unique tblNum
  index tblName-idx is unique tblName
.

define temp-table tt_usrIdxInfoX
  field xId     as integer
  field idxNum  as integer
  field idxName as character format "x(40)"		/* tblName.idxName					*/
  field idxRd   as int64 extent 2			/* [1] = previous value, [2] = incremental value	*/
  field idxCr   as int64 extent 2
  field idxDl   as int64 extent 2
  index idxNum-idx  is unique idxNum
  index idxName-idx is unique idxName
.

define temp-table tt_usrLOBInfoX no-undo
  field xId     as integer
  field lobNum  as integer
  field lobName as character format "x(20)"
  field lobRd   as int64 extent 2			/* [1] = previous value, [2] = incremental value	*/
  field lobCr   as int64 extent 2    
  field lobUp   as int64 extent 2
  field lobDl   as int64 extent 2
  index lobNum-idx  is unique lobNum
  index lobName-idx is unique lobName
.

define variable myUserNum     as integer no-undo.
define variable myUserid      as integer no-undo.

define variable tblBase       as integer no-undo.
define variable tblRange      as integer no-undo.

define variable idxBase       as integer no-undo.
define variable idxRange      as integer no-undo.

define variable lobBase       as integer no-undo.
define variable lobRange      as integer no-undo.

/* offsets into the _user* VSTs
 */

define variable firstUsrTblId as integer no-undo.
define variable lastUsrTblId  as integer no-undo.

define variable firstUsrIdxId as integer no-undo.
define variable lastUsrIdxId  as integer no-undo.

define variable firstUsrLobId as integer no-undo.
define variable lastUsrLOBId  as integer no-undo.



/* who am i? (this should probably be an input parameter)
 */

find dictdb._myConnect no-lock.

/* VSTs are keyed by "id", userNum is userId + 1 -- or, depending on your POV, userId is userNum - 1 
 *
 * in any event efficient VST access is always by the "id" field so make sure to use it!
 *
 */

myUserNum = _myConn-UserId.
myUserid  = _myConn-UserId + 1.

/* how many tables are there and what are the starting and ending offsets?
 *
 * -tablerangesize and -indexrangesize both default to 50, if you have more
 * than 50 tables or indexes the appropriate *rangesize parameter will need
 * to be increased
 *
 */
 
find first dictdb._tableStat no-lock.
tblBase = _tableStat-id.

find last _tableStat no-lock.
tblRange = _tableStat-id - tblBase + 1.

assign
  firstUsrTblId = myUserNum * tblRange
  lastUsrTblId  = firstUsrTblId + tblRange
  firstUsrTblId = firstUsrTblId + 1
.

/* how many indexes are there?  what are the starting and ending offsets?
 */

find first dictdb._indexStat no-lock.
idxBase = _indexStat-id.

find last _indexStat no-lock.
idxRange = _indexStat-id - idxBase + 1.

assign
  firstUsrIdxId = myUserNum * idxRange
  lastUsrIdxId  = firstUsrIdxId + idxRange
  firstUsrIdxId = firstUsrIdxId + 1
.

/*
 * message "myUserNum:" myUserNum " tblBase:" tblBase " tblRange:" tblRange " firstUsrTblId:" firstUsrTblId " lastUsrTblId:" lastUsrTblId.
 * message " myUserId:" myUserId  " idxBase:" idxBase " idxRange:" idxRange " firstUsrIdxId:" firstUsrIdxId " lastUsrIdxId:" lastUsrIdxId.
 * pause.
 */

/* LOB stats are not available prior to OE12
 */

&IF DECIMAL(SUBSTRING(PROVERSION,1,INDEX(PROVERSION,".") + 1)) >= 12.0 &THEN

find first dictdb._LOBStat no-lock.
LOBBase = recid( _LOBStat ).

find last _LOBStat no-lock.
LOBRange = recid( _LOBStat ).

assign
  firstUsrLobId = myUserNum * LOBRange
  lastUsrLOBId  = firstUsrLobId + LOBRange
  firstUsrLobId = firstUsrLobId + 1
.

&ENDIF

/* initialize user stats
 */

run getUStats ( output table tt_usrTblInfo by-reference, output table tt_usrIdxInfo by-reference, output table tt_usrLOBInfo by-reference ).

session:add-super-procedure( this-procedure ).

return.


/************************************************************/


/* get user table & index stats
 *
 * the first time this is run it will initialize the baseline (that is why it is run at startup).
 *
 */

procedure getUStats:

  define output parameter table for tt_usrTblInfo.
  define output parameter table for tt_usrIdxInfo.
  define output parameter table for tt_usrLOBInfo.

  for each dictdb._userTableStat no-lock where _UserTableStat-Id >= firstUsrTblId and _UserTableStat-Id <= lastUsrTblId and _userTableStat-num <> ?:

    find tt_usrTblInfoX where tt_usrTblInfoX.xId = _userTableStat-id no-error.

    if available tt_usrTblInfoX then
      assign

        tt_usrTblInfoX.tblRd[2] = _userTableStat-read   - tt_usrTblInfoX.tblRd[1]
        tt_usrTblInfoX.tblRd[1] = _userTableStat-read

        tt_usrTblInfoX.tblCr[2] = _userTableStat-create - tt_usrTblInfoX.tblCr[1]
        tt_usrTblInfoX.tblCr[1] = _userTableStat-create

        tt_usrTblInfoX.tblUp[2] = _userTableStat-update - tt_usrTblInfoX.tblUp[1]
        tt_usrTblInfoX.tblUp[1] = _userTableStat-update

        tt_usrTblInfoX.tblDl[2] = _userTableStat-delete - tt_usrTblInfoX.tblDl[1]
        tt_usrTblInfoX.tblDl[1] = _userTableStat-delete
      .
     else
      do:
        find first _file no-lock where _file-num = _userTableStat-num no-error.
        if not available _file then
          do:
            /* message "wtf? tbl...". */
            /* pause. */
            next.
          end.
        create tt_usrTblInfoX.
        assign
          tt_usrTblInfoX.xId      = _userTableStat-id
          tt_usrTblInfoX.tblNum   = _userTableStat-num
          tt_usrTblInfoX.tblName  = ( if available _file then _file-name else string( _userTableStat-num ))
          tt_usrTblInfoX.tblRd[1] = _userTableStat-read
          tt_usrTblInfoX.tblCr[1] = _userTableStat-create
          tt_usrTblInfoX.tblUp[1] = _userTableStat-update
          tt_usrTblInfoX.tblDl[1] = _userTableStat-delete
        .
      end.

  end.

  for each dictdb._userIndexStat no-lock where _UserIndexStat-id >= firstUsrIdxId and _userIndexStat-id <= lastUsrIdxId and _userIndexStat-num <> ?:

    find tt_usrIdxInfoX where tt_usrIdxInfoX.xId = _UserIndexStat-id no-error.
    if available tt_usrIdxInfoX then
      assign
        tt_usrIdxInfoX.idxRd[2]  = _userIndexStat-read   - tt_usrIdxInfoX.idxRd[1]
        tt_usrIdxInfoX.idxRd[1]  = _userIndexStat-read

        tt_usrIdxInfoX.idxCr[2]  = _userIndexStat-create - tt_usrIdxInfoX.idxCr[1]
        tt_usrIdxInfoX.idxCr[1]  = _userIndexStat-create

        tt_usrIdxInfoX.idxDl[2]  = _userIndexStat-delete - tt_usrIdxInfoX.idxDl[1]
        tt_usrIdxInfoX.idxDl[1]  = _userIndexStat-delete
      .
     else
      do:
        find first _index no-lock where _idx-num = _UserIndexStat-num no-error.
        if available _index then find first _file no-lock of _index no-error.
        if not available _index /* or not available _file */ then
          do:
            /* message "wtf? idx...". */
            /* pause. */
            next.
          end.
        create tt_usrIdxInfoX.
        assign
          tt_usrIdxInfoX.xId      = _userIndexStat-id
          tt_usrIdxInfoX.idxNum   = _UserIndexStat-num
          tt_usrIdxInfoX.idxName  = substitute( "&1.&2", ( if available _file then _file-name else "unknown" ), _index-name )
          tt_usrIdxInfoX.idxRd[1] = _userIndexStat-read
          tt_usrIdxInfoX.idxCr[1] = _userIndexStat-create
          tt_usrIdxInfoX.idxDl[1] = _userIndexStat-delete
        .
      end.

  end.

&IF DECIMAL(SUBSTRING(PROVERSION,1,INDEX(PROVERSION,".") + 1)) >= 12.0 &THEN

/*
 *   message "LOBs:" firstUsrLobId lastUsrLOBId.
 *   pause.
 */

  for each dictdb._userLOBStat no-lock where _UserLOBStat-Id >= firstUsrLobId and _UserLOBStat-Id <= lastUsrLOBId and _userLOBStat-num <> ?:

    find tt_usrLOBInfoX where tt_usrLOBInfoX.xId = _userLOBStat-id no-error.

    if available tt_usrLOBInfoX then
      assign

        tt_usrLOBInfoX.lobRd[2] = _userLOBStat-read   - tt_usrLOBInfoX.lobRd[1]
        tt_usrLOBInfoX.lobRd[1] = _userLOBStat-read

        tt_usrLOBInfoX.lobCr[2] = _userLOBStat-create - tt_usrLOBInfoX.lobCr[1]
        tt_usrLOBInfoX.lobCr[1] = _userLOBStat-create

        tt_usrLOBInfoX.lobUp[2] = _userLOBStat-update - tt_usrLOBInfoX.lobUp[1]
        tt_usrLOBInfoX.lobUp[1] = _userLOBStat-update

        tt_usrLOBInfoX.lobDl[2] = _userLOBStat-delete - tt_usrLOBInfoX.lobDl[1]
        tt_usrLOBInfoX.lobDl[1] = _userLOBStat-delete
      .
     else
      do:

        find _file  no-lock where _file._file-num = _userLOBStat-Table no-error.
        find _field no-lock where _field._file-recid = recid( _file ) and _field._fld-stlen = _userLOBStat-num no-error.

        create tt_usrLOBInfoX.
        assign
          tt_usrLOBInfoX.xId      = _userLOBStat-id
          tt_usrLOBInfoX.lobNum   = _userLOBStat-num
          tt_usrLOBInfoX.lobName  = substitute( "&1.&2", ( if available _file then _file-name else string( _userLOBStat-table )), ( if available _field then _field-name else string( _userLOBStat-num )))
          tt_usrLOBInfoX.lobRd[1] = _userLOBStat-read
          tt_usrLOBInfoX.lobCr[1] = _userLOBStat-create
          tt_usrLOBInfoX.lobUp[1] = _userLOBStat-update
          tt_usrLOBInfoX.lobDl[1] = _userLOBStat-delete
        .

      end.

  end.

&ENDIF

  /* copy the incremental values to a more user friendly TT
   */

  for each tt_usrTblInfoX:
    find tt_usrTblInfo where tt_usrTblInfo.tblName = tt_usrTblInfoX.tblName no-error.
    if not available tt_usrTblInfo then create tt_usrTblInfo.
    assign
      tt_usrTblInfo.tblName = tt_usrTblInfoX.tblName
      tt_usrTblInfo.tblRd   = tt_usrTblInfoX.tblRd[2]
      tt_usrTblInfo.tblCr   = tt_usrTblInfoX.tblCr[2]
      tt_usrTblInfo.tblUp   = tt_usrTblInfoX.tblUp[2]
      tt_usrTblInfo.tblDl   = tt_usrTblInfoX.tblDl[2]
    .
  end.

  for each tt_usrIdxInfoX:
    find tt_usrIdxInfo where tt_usrIdxInfo.idxName = tt_usrIdxInfoX.idxName no-error.
    if not available tt_usrIdxInfo then create tt_usrIdxInfo.
    assign
      tt_usrIdxInfo.idxName = tt_usrIdxInfoX.idxName
      tt_usrIdxInfo.idxRd   = tt_usrIdxInfoX.idxRd[2]
      tt_usrIdxInfo.idxCr   = tt_usrIdxInfoX.idxCr[2]
      tt_usrIdxInfo.idxDl   = tt_usrIdxInfoX.idxDl[2]
    .
  end.

  for each tt_usrLOBInfoX:
    find tt_usrLOBInfo where tt_usrLOBInfo.lobName = tt_usrLOBInfoX.lobName no-error.
    if not available tt_usrLOBInfo then create tt_usrLOBInfo.
    assign
      tt_usrLOBInfo.lobName = tt_usrLOBInfoX.lobName
      tt_usrLOBInfo.lobRd   = tt_usrLOBInfoX.lobRd[2]
      tt_usrLOBInfo.lobCr   = tt_usrLOBInfoX.lobCr[2]
      tt_usrLOBInfo.lobUp   = tt_usrLOBInfoX.lobUp[2]
      tt_usrLOBInfo.lobDl   = tt_usrLOBInfoX.lobDl[2]
    .
  end.

  return.

end.
