/* util/pt3clean.p
 *
 * truncate logs and remove leftover files in TMPDIR and LOGDIR
 *
 * mark b 4/20/20
 */

define variable PROTOP as character no-undo.
define variable inDir  as character no-undo.
define variable inFile as character no-undo.
define variable osCmd  as character no-undo.

{lib/protop.i}

run lib/protoplib.p persistent.							/* load protop infrastructure library		*/
run lib/protop-cfg.p.

/* PROPATH = pt_tmpdir + "," + pt_logdir + "," + PROPATH.	*/		/* file-info uses propath to pull full-pathname */

if lookup( pt_tmpdir, propath ) <= 0 then
  propath = propath + "," + pt_tmpdir.

if lookup( pt_logdir, propath ) <= 0 then
  propath = propath + "," + pt_logdir.


function deleteFile returns logical ( fileName as character ):

  file-info:file-name = fileName.

  if file-info:full-pathname = ? then
    return false.
   else
    os-delete value( file-info:full-pathname ).

  return true.    			/* nothing below is actually paying attention, but that might change some day! */
   
end.


assign
  PROTOP = os-getenv( "PROTOP" ) 
  inDir = pt_tmpdir
.

if protop = ? then return.

input from os-dir( inDir ).       /* TMPDIR */

repeat:

    import inFile.

    if inFile matches "*.flg" then deleteFile( inFile ).

end.

input close.

/* now that the sub-service flags are gone, wait 10 seconds for their absence to register and services to be shut down 
 */

pause 10.

/* TMPDIR - remove all the files listed below which tend to accumulate over time
 */

input from os-dir( inDir ).       /* TMPDIR */

repeat:

    import inFile.

    if       inFile matches "dbmonitor.*" then deleteFile( inFile ).

     else if inFile matches "dbi*" then deleteFile( inFile ).			/* this section is the main junk that proliferates		*/
     else if inFile matches "DBI*" then deleteFile( inFile ).
     else if inFile matches "rcd*" then deleteFile( inFile ).
     else if inFile matches "lbi*" then deleteFile( inFile ).
     else if inFile matches "srt*" then deleteFile( inFile ).
     else if inFile matches "protrace*" then deleteFile( inFile ).
     else if inFile matches "procore*" then deleteFile( inFile ).
     else if inFile matches "core*" then deleteFile( inFile ).

     else if inFile matches "*.debug" then deleteFile( inFile ).		/* these grow and grow and grow - but they don't proliferate	*/
     else if inFile matches "*.err" then deleteFile( inFile ).
     else if inFile matches "*.e2" then deleteFile( inFile ).

     else if inFile matches "*.tar" then deleteFile( inFile ).			/* miscellaneous junk						*/
     else if inFile matches "*.tar.gz" then deleteFile( inFile ).
     else if inFile matches "*.zip" then deleteFile( inFile ).

     else if inFile matches "mount.*" then deleteFile( inFile ).		/* cache files with "mount" output				*/

end.

input close.

/* truncate the log files in LOGDIR
 */

inDir = pt_logdir.

input from os-dir( inDir ).       /* LOGDIR */

repeat:

   import inFile.

   if inFile matches "*.log" then deleteFile( inFile ).

end.

input close.

/* remove unneeded files from PROTOP directory */

inDir = PROTOP.

input from os-dir( inDir ).       /* PROTOP directory */

repeat:

   import inFile.

   if       inFile matches "protrace*" then deleteFile( inFile ).		/* this section is the main junk that proliferates		*/
    else if inFile matches "procore*" then deleteFile( inFile ).
    else if inFile matches "core*" then deleteFile( inFile ).
    else if inFile matches "*.ped" then deleteFile( inFile ).

    else if inFile = "pt3inst.log" then deleteFile( inFile ).			/* these grow and grow and grow - but they don't proliferate	*/
    else if inFile = "pt3inst.err" then deleteFile( inFile ).

    else if inFile matches "*.tar" then deleteFile( inFile ).			/* miscellaneous junk						*/
    else if inFile matches "*.zip" then deleteFile( inFile ).


end.

input close.

/* restart dbmonitor */

case OPSYS:
   when "unix" then do:
      osCmd = PROTOP + "/bin/dbmonitor.sh > " + PROTOP + "/log/dbmonitor.cron.log 2>&1".
      os-command no-wait value( osCmd ).
   end.
end case.

quit.


/*** finis ***/
